% Demo - fuzzy logic operators.

% (c) Robert Babuska, March 1994

echo off
% Reset things before we get started.
clear
clc;clf;
format
echo on
%        SET-THEORETIC OPERATIONS
%
% This part introduces the basic operations with fuzzy sets. Classical union,
% intersection and complement of ordinary sets can be extended to fuzzy sets
% by the following formulas proposed originally by Zadeh:

%        A or B  = max(Ax,Bx)
%        A and B = min(Ax,Bx)
%        not A   = 1 - Ax;

% Press any key to continue...
pause
clc
% Example: On an interval of real numbers (-1...1) we define fuzzy sets A and B:

X = -1:0.01:1;
A = mgrade(X,[3 -0.8 -0.1 -0.1 0.3]);
B = mgrade(X,[3 -0.4  0.3  0.3 0.9]);

% The following fuzzy sets are union, intersection and complement of A and B
% respectively:

AorB = or_za(A,B);
AandB = and_za(A,B);
NotA = not_za(A);
NotB = not_za(B);

% Press any key to continue...
pause
clc
% Let us plot these fuzzy sets ...

echo off; clc; clf;
subplot(221); plot(X,A,'y',X,B,'c','linewidth',2); hold on;
plot(X,AorB,'r','LineWidth',2); hold off; title('A or B'); drawnow;
subplot(222); plot(X,A,'y',X,B,'c','linewidth',2); hold on;
plot(X,AandB,'r','LineWidth',2); hold off; title('A and B'); drawnow;
subplot(223); plot(X,A,'y',X,NotA,'r','linewidth',2); title('Not A'); drawnow;
subplot(224); plot(X,B,'c',X,NotB,'r','linewidth',2); title('Not B'); drawnow;
echo on; clc;

% Legend:

% Top-left     ... yellow - fuzzy set A, cyan - fuzzy set B, red - UNION
% Top-right    ... yellow - fuzzy set A, cyan - fuzzy set B, red - INTERSECTION
% Bottom-left  ... yellow - fuzzy set A, red - COMPLEMENT of A
% Bottom-right ... cyan   - fuzzy set B, red - COMPLEMENT of B

% Press any key to continue...
pause
clc
% ALTERNATIVE OPERATORS

% Other operators can be defined for union and intersection,
% for example the probabilistic operators:

% A or B  = Ax + Bx - Ax.Bx
% A and B = Ax.Bx

% Let us compare them to the Zadeh's operators:

AorBp  = or_pr(A,B);
AandBp = and_pr(A,B);

% Press any key to continue...
pause

echo off; clc; clf;
subplot(221); plot(X,A,'y',X,B,'c','linewidth',2); hold on;
plot(X,AorB,'r','LineWidth',2); hold off; title('Zadeh''s OR'); drawnow;
subplot(222); plot(X,A,'y',X,B,'c','linewidth',2); hold on;
plot(X,AorBp,'r','LineWidth',2); hold off; title('Probability OR'); drawnow;
subplot(223); plot(X,A,'y',X,B,'c','linewidth',2); hold on;
plot(X,AandB,'r','LineWidth',2); hold off; title('Zadeh''s AND'); drawnow;
subplot(224); plot(X,A,'y',X,B,'c','linewidth',2); hold on;
plot(X,AandBp,'r','LineWidth',2); hold off; title('Probability AND'); drawnow;
echo on

% Press any key to continue...
pause
clc
% For the union and intersection we can use also the bold union
% and bold intersection (due to Lukasiewicz):

% A or B  = min(1,Ax + Bx)
% A and B = max(0,Ax + Bx -1)

% Let us compare them to the Zadeh's operators:

AorBl = or_lu(A,B);
AandBl = and_lu(A,B);

% Press any key to continue...
pause

echo off; clc; clf;
subplot(221); plot(X,A,'y',X,B,'c','linewidth',2); hold on;
plot(X,AorB,'r','LineWidth',2); hold off; title('Zadeh''s OR'); drawnow;
subplot(222); plot(X,A,'y',X,B,'c','linewidth',2); hold on;
plot(X,AorBl,'r','LineWidth',2); hold off; title('Lukasiewicz''s OR'); drawnow;
subplot(223); plot(X,A,'y',X,B,'c','linewidth',2); hold on;
plot(X,AandB,'r','LineWidth',2); hold off; title('Zadeh''s AND'); drawnow;
subplot(224); plot(X,A,'y',X,B,'c','linewidth',2); hold on;
plot(X,AandBl,'r','LineWidth',2); hold off; title('Lukasiewicz''s AND'); drawnow;
echo on

% Press any key to continue...
pause
clc
% T-NORMS and T-CONORMS

% Functions for intersection are generally called T-norms and functions for
% union T-conorms (or S-norms). Note that every T-norm is smaller than MIN
% and every T-conorm is bigger than MAX. Various families of
% parametric t-norms and t-conorms exist (e.g. due to Dubois and Prade,
% Yager, Dombi, etc.). Operators between MIN and MAX are called aggregation
% or averaging operators (e.g. mean, Delft operator).

% Press any key to continue...
pause
clc
% There are also two alternative types of complements, due to Sugeno and Yager:

% Sugeno: not A = (1 - Ax)/(1 + lambda.Ax)
% Yager:  not A = (1 - Ax^w)^(1/w)

% Note that both operators are parametric (the result depends on the
% choice of parameters lambda and w). Let us compare them to the Zadeh's
% operator:

NotAs = not_su(A,-0.5);
NotAy = not_ya(A,0.5);

% Press any key to continue...
pause
echo off; clc; clf;
subplot(221); plot(X,A,'y',X,NotA,'r','linewidth',2); title('Zadeh''s NOT'); drawnow;
subplot(222); plot(X,A,'y',X,NotAs,'r','linewidth',2); title('Sugeno''s NOT'); drawnow;
subplot(223); plot(X,A,'y',X,NotA,'r','linewidth',2); title('Zadeh''s NOT'); drawnow;
subplot(224); plot(X,A,'y',X,NotAy,'r','linewidth',2); title('Yager''s NOT'); drawnow;
echo on

% Press any key to continue...
pause
clc
% Finally let us show how the union and intersection operators work in
% Cartesian product space.

X = -1:0.05:1;
A = mgrade(X,[3 -0.8 -0.1 -0.1 0.3]);
B = mgrade(X,[3 -0.4  0.3  0.3 0.9]);
C = and_za(A',B);
D = or_za(A',B);

% Press any key to continue...
pause
clc
echo off; clc; clf;
subplot(221); plot(X,A); title('A'); drawnow;
subplot(222); mesh(X,X,C); title('A and B'); drawnow;
subplot(223); contour(X,X,C); title('A and B'); drawnow;
subplot(224); plot(X,B); title('B'); drawnow;
echo on

% Press any key to continue...
pause
clc

echo off; clc; clf;
subplot(221); plot(X,A); title('A'); drawnow;
subplot(222); mesh(X,X,D); title('A or B'); drawnow;
subplot(223); contour(X,X,D); title('A or B'); drawnow;
subplot(224); plot(X,B); title('B'); drawnow;
echo on

% Press any key to quit...
pause
echo off; clc;

