% Demo - fuzzy modeling.

% (c) Robert Babuska, March 1994

echo off
% Reset things before we get started.
clear;clc;clf;
figure(1); clf;
echo on

% For simple nonlinear systems, a Takagi-Sugeno fuzzy model can be
% developed  by defining equally spaced and shaped membership
% functions for those variables that influence the nonlinearity of
% the process. These variables are used in the antecedents of the
% rules. The consequent parameters can  be estimated from data using
% least squares. In literature, this approach  is usually denoted as
% template-based modeling. An example illustrates this method.

% Press any key to continue...
pause;
clc
% This example shows the use of function SUGLMS and SUGVAL for
% the modeling of dynamical systems. A simple nonlinear first-order
% system is considered. The system is defined by the following
% difference equation:

%      y(k+1) = y(k) + exp(-3*abs(y(k)))*u(k)

% Press any key to continue...
pause;
clc
% This system exhibits dynamics similar to the pH neutralization
% process. A set of input-output data was generated in simulation and
% stored on disk in file phdata.mat. Let us load this file and plot the
% system input and output. We also add normally distributed "sensor"
% noise to the output. Notice that the data is not evenly distributed
% in the output domain. Since the system has high gain around zero,
% only a few data point are available in this region.

% Press any key to continue...
pause;
clc
load phdata.mat /ascii;
t = phdata(:,1);
uph = phdata(:,2);
yph = phdata(:,3);

randn('seed',0);                        % to get reproducible results
yph = yph + 0.05*randn(length(yph),1);  % noise

echo off
clf;
subplot(211); plot(t,uph); ylabel('Input u(k)'); xlabel('Time')
subplot(212); plot(t,yph,'.',t,yph,'markersize',10);
ylabel('Output y(k)');  xlabel('Time')
axis([0 300 -1.5 1.5])

echo on
% Press any key to continue...
pause;
clc;

% Knowing the order of the ``true'' system and using the fact that
% the nonlinear gain depends on the output, the following TS model
% structure is chosen:

% If y(k) is SMALL  then y(k+1) = a_small*y(k) +  b_small*u(k)
% If y(k) is MEDIUM then y(k+1) = a_medium*y(k) + b_medium*u(k)
% etc.

% Press any key to continue...
pause;
clc;

% The consequent parameters a_small, a_medium, ... can be estimated
% by least squares. In order to compute the estimates, we prepare
% the regression matrix [y(k) u(k)] and the output vector y(k+1):

[r,y] = regres([yph uph],1,1,1);
y1 = y; y = r(:,1); u = r(:,2);
inputs = [y u];

% Press any key to continue...
pause;
clc;

% The delayed system output y is partitioned in 7 equidistant
% triangular fuzzy sets and membership grades of the data are
% calculated.

mfy = mfequ([-1.5 -1.4 1.4 1.5],1,7);
dof = mgrade(y',mfy)';

echo off
figure(1); clf
plot(-1.5:0.01:1.5,mgrade(-1.5:0.01:1.5,mfy),'-','linewidth',2);
set(gca,'box','off')
axis([-1.5 1.5 0 1])
xlabel('y(k)'); ylabel('Membership grade')
title('Membership functions');
grid;
set(gca,'position',[.13 .31 .775 .5]);

echo on
% Press any key to continue...
pause;
clc;

% The consequent parameters are calculated as a least-squares solution,
% using function SUGLMS.

[p ym yl ylm] = suglms(inputs,y1,dof);

% Press any key to continue...
pause;
clc;

% Let us plot the fit on the identification data.

% The fit is very good which is usual and which
% does not say much about the quality of the model.
% Let us now simulate the data from the input only.

echo off
figure(1); clf;
plot([y1 ym]); drawnow;
xlabel('Time'); ylabel('Output');
title('1SAP on ID data');
axis([0 300 -1.5 1.8]);
legend('system','model');

echo on
% Press any key to continue...
pause;
clc;

echo off
figure(1); clf; plot((1:length(yph)),yph); 
xlabel('Time'); ylabel('Output'); drawnow; hold on; 
yme = yph; 
clear dofe
for k = 2 : size(uph,1),
  yme(k) = sugval(p,[yme(k-1) uph(k-1)],mgrade(yme(k-1),mfy)',yme(k-1));
  plot([k-1 k],[yme(k-1) yme(k)],'m-','EraseMode','none'); drawnow;
end;
hold off

clf; plot([yph yme],'EraseMode','none');
xlabel('Time'); ylabel('Output'); 
axis([0 300 -1.5 1.8]); legend('system','model');

echo on
% Press any key to continue...
pause;
clc;

% For complete validation, another data set should be generated
% and the model should be tested on this data. Here we only compare
% the nonlinear surface of the original system to the surface of the
% identified model. First let us generate surface of the system.

echo off
udom = min(u):0.1:max(u);
ydom = min(y):0.1:max(y);
[uu,yy] = meshgrid(udom,ydom);
uy = [yy(:) uu(:)];

figure(1); clf; subplot(211)
mesh(uu,yy,yy+exp(-3*abs(yy)).*uu);
title('Original surface');
axis([-1 1 -2 2 -1.5 1.5])

echo on
% Press any key to continue...
pause;
clc;

% And now the surface of the Takagi-Sugeno fuzzy model.
% Also the data points "measured" on the system are shown
% on the surface. Notice that a reasonable approximation
% of the original surface is obtained from a relatively
% sparse data set. This is achieved thanks to a suitably
% chosen structure of the model and by using some prior
% knowledge to select the antecedent variables.

echo off
zz = yy;
zz(:) = sugval(p,uy,mgrade(yy(:)',mfy)');
subplot(212);
mesh(uu,yy,zz);
title('Model surface');
hold on;
plot3(u,y,y1,'.','markersize',9);
hold off
axis([-1 1 -2 2 -1.5 1.5])

echo on
% Press any key to continue...
pause;
clc;

% Finally, let us plot the consequent parameters against
% the corest of the antecedent membership functions.
% The dependence of the consequent parameters on the antecedent
% variable approximates quite accurately the system's nonlinearity,
% which gives the model certain transparency. The parameters
% indicate the strong nonlinearity of the u-part (the nonlinear gain)
% and the linearity of the y-part (linear dynamics) of the system.

echo off
figure(1); clf;
plot(mfy(:,3),p,mfy(:,3),p,'.',-1.5:0.1:1.5,exp(-3*abs(-1.5:0.1:1.5)),'markersize',20)
grid;
xlabel('y'); ylabel('Parameters a,b')
title('Consequent parameters');
axis([-1.5 1.5 0 1.1])

echo on
% Press any key to continue...
pause; 
clc;

% Now try to experiment with the data by choosing a another 
% type and number of the membership functions. Open the file
% phdemo.m and have a look at the implementation of the different
% identification steps. You can obtain help on the individual
% functions by typing help funtion_name in the MATLAB workspace,
% for instance help mgrade.

echo on
% Press any key to stop...
pause;
echo off; clc;


