function txt = rls2txt(rls,mfs,inputs,outputs,nlab,labels,filename);
% RLS2TXT writes the rule base coded in a matrix into a string
% or file. TXT = RLS2TXT(RLS,MFS,INPUTS, OUTPUTS, NLAB, LABELS,FILENAME)
%
% Inputs:     RLS       ...  rule-base matrix
%	      MFS	...  membership functions matrix (can be empty)
%             INPUTS    ...  input labels
%             OUTPUTS   ...  output labels
%             NLAB      ...  number of labels for all inputs and outputs
%                            i.e. [i1; i2; i3; ... o1; o2; ...]
%             LABELS    ...  input and output labels concatenated in one
%                            matrix, i.e. [Input1_Label1; Input1_Label2;
%                                                       ...
%                                          Input2_Label1; Input2_Label2;
%                                                       ...
%                                          Output1_Label1; Output1_Label2;
%                                                       ...               ]
%             FILENAME  ...  file name (optional)
% Output:     TXT       ...  text matrix
%
% RLS2TXT is an inverse function to TXT2RLS, i.e. after
%
%       [rls,mfs,inputs,outputs,nlab,labels] = txt2rls('rbase1.rb');
%       rls2txt(rls,mfs,inputs,outputs,nlab,labels,'rbase2.rb');
%
% rule-base file rbase2.rb contains the same rule-base as rbase1.rb,
% except comments and formatting.
%
% See also TXT2RLS, SAMPLE.RB, MFSET, MFGET, INFER

% (c) Robert Babuska, December 1993


% directives and keywords
inpstr = '#input'; outstr  = '#output'; rulstr  = '#rules';
mfdstr = '#mf';
IfStr   = ' If '; ThenStr = ' then '; IsStr   = ' is ';
AndStr  = ' and '; NotStr  = ' not '; RuleEnd = '; ';
CSepStr = ', ';

mfnum = size(mfs,1);

[ninps,dummy] = size(inputs);
[nrules,nio] = size(rls);
w = rls(:,1); rls = rls(:,2:nio);
nouts = nio - ninps - 1;
nlab = nlab(:);
labind = [0; cumsum(nlab)];

txt = [];
for i = 1 : ninps,
  txt = str2mat(txt,' ',[inpstr ' ' inputs(i,:)]);
  for j = labind(i)+1 : labind(i+1),
    if j <= mfnum,
        mf = ['     ' mfdstr sprintf('%8g',mfs(j,:))];
    else mf = [];
    end;
    txt = str2mat(txt,['        ' labels(j,:) mf]);
  end;
end;
for i = 1 : nouts,
  txt = str2mat(txt,' ',[outstr ' ' outputs(i,:)]);
  for j = labind(ninps+i)+1 : labind(ninps+i+1),
    if j <= mfnum,
        mf = ['     ' mfdstr sprintf('%8g',mfs(j,:))];
    else mf = [];
    end;
    txt = str2mat(txt,['        ' labels(j,:) mf]);
  end;
end;

txt = str2mat(txt,' ',rulstr);
for k = 1 : nrules,
  if any(w ~= 1), weight = sprintf('%4.2f',w(k));
  else weight = []; end
  rule = [' ' weight IfStr];
  j = 0;
  for i = 1 : ninps
    if rls(k,i) ~= 0,
      j = j + 1;
      if j > 1, rule = [rule AndStr]; end;
      if rls(k,i) < 0; label = [NotStr labels(abs(rls(k,i)),:)];
      else label = labels(rls(k,i),:); end;
      rule = [rule inputs(i,:) IsStr label];
    end;
    if i == ninps, rule = [rule ThenStr]; end;
  end;
  j = 0;
  for i = 1 : nouts
    if rls(k,ninps+i) ~= 0 | nlab(ninps+i) == 0,
      j = j + 1;
      if j > 1, rule = [rule(1:length(rule)-1) CSepStr]; end;
      if nlab(ninps+i) ~= 0, cons = labels(rls(k,ninps+i),:);
      else cons = sprintf('%f',rls(k,ninps+i)); end;
      rule = [rule outputs(i,:) IsStr cons ' '];
    end;
  end;
  ind = find(rule == ' ');
  li = length(ind);
  rule(ind(find(ind(2:li)-ind(1:li-1) == 1))) = [];   % remove multiple blanks
  txt = str2mat(txt,[rule(1:length(rule)-1) RuleEnd]);
end;
txt(1:2,:) = [];

if nargin > 6,
  fid = fopen(filename,'w');
  if fid == -1, error(['Error opening file ' filename]); end;
  for i = 1 : size(txt,1),
    fprintf(fid,'%s\n',deblank(txt(i,:)));
  end;
  fclose(fid);
end;

